==> welltested-2.1.4.tar.gz: lib/src/utilities/cache_manager.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:azure_application_insights/azure_application_insights.dart';
import 'package:path/path.dart' as p;
import 'package:welltested/src/utilities/logging.dart';
import 'package:welltested/src/utilities/path_utils.dart';
import 'package:welltested/src/utilities/telemetry_client.dart';
import 'package:welltested/src/utilities/uuid.dart';

enum TestExistStatus {
  doesNotExist,
  outdatedCodeExists,
  onlyRequestExists,
  generationExists
}

//TODO: Only store relative file paths
class CacheManager {
  static final CacheManager _singleton = CacheManager._internal();

  factory CacheManager() {
    return _singleton;
  }

  CacheManager._internal();

  final file = File(p.join(PathUtils.currentPath, '.dart_tool', 'pub',
      'welltested', 'welltested.json'));
  final List<CodeCache> generationsCache = [];

  Future<void> initialize() async {
    if (file.existsSync()) {
      final content = await file.readAsString();
      if (content.isEmpty) return;
      try {
        final generations =
            jsonDecode(content)['generations'] as Map<String, dynamic>;
        generationsCache.addAll(
            generations.entries.map((e) => CodeCache.fromJson(e.key, e.value)));
      } catch (e, stackTrace) {
        wtTelemetry.trackError(
            severity: Severity.warning, error: e, stackTrace: stackTrace);
        wtLog.warning(
            "Discrepancy found in the cache file. Clearing existing cache.");
        await file.writeAsString(jsonEncode({'generations': {}}));
      }
    }
  }

  Future<void> closeAndSettle() async {
    await file.create(recursive: true);
    final fileData = {
      'generations': {}..addEntries(
          generationsCache.map((e) => MapEntry(e.codeHash, e.toJson())))
    };
    await file.writeAsString(jsonEncode(fileData));
  }

  Future<void> cacheTest(
      {required String code,
      required String codeFilePath,
      required String test,
      required String testFilePath,
      required String functionName,
      String? className,
      bool saved = false}) async {
    codeFilePath = PathUtils.relatetiveLibFilePath(codeFilePath);
    testFilePath = PathUtils.relativeTestFilePath(testFilePath);
    final codeHash = uniqueCodeHex(code);
    final testHash = uniqueCodeHex(test);
    generationsCache.removeWhere((element) =>
        element.codeHash == codeHash ||
        (element.codeFilePath == codeFilePath &&
            element.className == className &&
            element.methodName == functionName));
    generationsCache.add(CodeCache(
        codeHash: codeHash,
        codeFilePath: codeFilePath,
        testHash: testHash,
        testFilePath: testFilePath,
        methodName: functionName,
        className: className,
        isSaved: saved));
  }

  Future<bool> checkIfTestIsSaved(
      String code, String filePath, String functionName,
      {String? className}) async {
    filePath = PathUtils.relatetiveLibFilePath(filePath);
    final codeHash = uniqueCodeHex(code);
    final matchingIndex = generationsCache.indexWhere((element) =>
        element.codeHash == codeHash &&
        element.codeFilePath == filePath &&
        element.className == className &&
        element.methodName == functionName);

    if (matchingIndex != -1) {
      final currentGeneration = generationsCache[matchingIndex];
      final testContent = await File(
              PathUtils.absouteTestFilePath(currentGeneration.testFilePath))
          .readAsString();
      final testHash = uniqueCodeHex(testContent);
      if (currentGeneration.testHash == testHash && currentGeneration.isSaved)
        return true; //yes, if test with same code is already saved.
    }

    return false;
  }

  Future<TestExistStatus> testExistStatus(
      String code, String filePath, String functionName,
      {String? className}) async {
    final testFileExists = await PathUtils.correspondingPermanentTestFile(
            functionName, File(filePath))
        .exists();

    filePath = PathUtils.relatetiveLibFilePath(filePath);
    final matchingFunctions = generationsCache.where((element) =>
        element.codeFilePath == filePath &&
        element.className == className &&
        element.methodName == functionName);

    if (matchingFunctions.isEmpty) {
      if (testFileExists) return TestExistStatus.outdatedCodeExists;
      return TestExistStatus.doesNotExist;
    }
    ;
    final exactMatch = matchingFunctions
        .where((element) => element.codeHash == uniqueCodeHex(code));
    if (exactMatch.isEmpty) return TestExistStatus.outdatedCodeExists;
    if (!testFileExists) return TestExistStatus.onlyRequestExists;
    return TestExistStatus.generationExists;
  }
}

class CodeCache {
  final String codeHash;
  final String codeFilePath;
  final String testHash;
  final String testFilePath;
  final String? className;
  final String methodName;
  final bool isSaved;

  CodeCache({
    required this.codeHash,
    required this.codeFilePath,
    required this.testHash,
    required this.testFilePath,
    required this.methodName,
    this.isSaved = false,
    this.className,
  });

  factory CodeCache.fromJson(
    String codeHash,
    Map<String, dynamic> json,
  ) =>
      CodeCache(
          codeHash: codeHash,
          codeFilePath: json['codeFilePath'],
          testHash: json['testHash'],
          testFilePath: json['testFilePath'],
          className: json['className'],
          methodName: json['functionName'],
          isSaved: json['saved']);

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['codeFilePath'] = codeFilePath;
    data['testHash'] = testHash;
    data['testFilePath'] = testFilePath;
    data['functionName'] = methodName;
    data['saved'] = isSaved;
    if (className != null) data['className'] = className;

    return data;
  }
}

==> welltested-2.2.0.tar.gz: lib/src/utilities/cache_manager.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:azure_application_insights/azure_application_insights.dart';
import 'package:path/path.dart' as p;
import 'package:welltested/src/utilities/logging.dart';
import 'package:welltested/src/utilities/path_utils.dart';
import 'package:welltested/src/utilities/telemetry_client.dart';
import 'package:welltested/src/utilities/uuid.dart';

enum TestExistStatus {
  doesNotExist,
  outdatedCodeExists,
  onlyRequestExists,
  generationExists
}

//TODO: Only store relative file paths
class CacheManager {
  static final CacheManager _singleton = CacheManager._internal();

  factory CacheManager() {
    return _singleton;
  }

  CacheManager._internal();

  final file = File(p.join(PathUtils.currentPath, '.dart_tool', 'pub',
      'welltested', 'welltested.json'));
  final List<CodeCache> generationsCache = [];

  Future<void> initialize() async {
    if (file.existsSync()) {
      final content = await file.readAsString();
      if (content.isEmpty) return;
      try {
        final generations =
            jsonDecode(content)['generations'] as Map<String, dynamic>;
        generationsCache.addAll(
            generations.entries.map((e) => CodeCache.fromJson(e.key, e.value)));
      } catch (e, stackTrace) {
        wtTelemetry.trackError(
            severity: Severity.warning, error: e, stackTrace: stackTrace);
        wtLog.warning(
            "Discrepancy found in the cache file. Clearing existing cache.");
        await file.writeAsString(jsonEncode({'generations': {}}));
      }
    }
  }

  Future<void> closeAndSettle() async {
    await file.create(recursive: true);
    final fileData = {
      'generations': {}..addEntries(
          generationsCache.map((e) => MapEntry(e.codeHash, e.toJson())))
    };
    await file.writeAsString(jsonEncode(fileData));
  }

  Future<void> cacheTest(
      {required String code,
      required String codeFilePath,
      required String test,
      required String testFilePath,
      required String functionName,
      String? className,
      bool saved = false}) async {
    codeFilePath = PathUtils.relatetiveLibFilePath(codeFilePath);
    testFilePath = PathUtils.relativeTestFilePath(testFilePath);
    final codeHash = uniqueCodeHex(code);
    final testHash = uniqueCodeHex(test);
    generationsCache.removeWhere((element) =>
        element.codeHash == codeHash ||
        (element.codeFilePath == codeFilePath &&
            element.className == className &&
            element.methodName == functionName));
    generationsCache.add(CodeCache(
        codeHash: codeHash,
        codeFilePath: codeFilePath,
        testHash: testHash,
        testFilePath: testFilePath,
        methodName: functionName,
        className: className,
        isSaved: saved));
  }

  Future<void> cacheWidgetTest(
       {required String code,
      required String codeFilePath,
      required String test,
      required String testFilePath,
      required String widgetName,
      bool saved = false}
  )async{
    codeFilePath = PathUtils.relatetiveLibFilePath(codeFilePath);
    testFilePath = PathUtils.relativeTestFilePath(testFilePath);
    final codeHash = uniqueCodeHex(code);
    final testHash = uniqueCodeHex(test);
    generationsCache.removeWhere((element) =>
        element.codeHash == codeHash ||
        (element.codeFilePath == codeFilePath &&
            element.widgetName == widgetName));
    generationsCache.add(CodeCache(
        codeHash: codeHash,
        codeFilePath: codeFilePath,
        testHash: testHash,
        testFilePath: testFilePath,widgetName:widgetName ,
        isSaved: saved));
  }

  Future<bool> checkIfTestIsSaved(
      String code, String filePath, String functionName,
      {String? className}) async {
    filePath = PathUtils.relatetiveLibFilePath(filePath);
    final codeHash = uniqueCodeHex(code);
    final matchingIndex = generationsCache.indexWhere((element) =>
        element.codeHash == codeHash &&
        element.codeFilePath == filePath &&
        element.className == className &&
        element.methodName == functionName);

    if (matchingIndex != -1) {
      final currentGeneration = generationsCache[matchingIndex];
      final testContent = await File(
              PathUtils.absouteTestFilePath(currentGeneration.testFilePath))
          .readAsString();
      final testHash = uniqueCodeHex(testContent);
      if (currentGeneration.testHash == testHash && currentGeneration.isSaved)
        return true; //yes, if test with same code is already saved.
    }

    return false;
  }

  Future<bool> checkIfWidgetTestSaved(
      String code, String filePath, String widgetName) async {
    filePath = PathUtils.relatetiveLibFilePath(filePath);
    final codeHash = uniqueCodeHex(code);
    final matchingIndex = generationsCache.indexWhere((element) =>
        element.codeHash == codeHash &&
        element.codeFilePath == filePath &&
        element.widgetName == widgetName);

    if (matchingIndex != -1) {
      final currentGeneration = generationsCache[matchingIndex];
      final testContent = await File(
              PathUtils.absouteTestFilePath(currentGeneration.testFilePath))
          .readAsString();
      final testHash = uniqueCodeHex(testContent);
      if (currentGeneration.testHash == testHash && currentGeneration.isSaved)
        return true; //yes, if test with same code is already saved.
    }

    return false;
  } 

  Future<TestExistStatus> widgetTestExistStatus(
      String code,
    String filePath,
    String widgetName,  
  ) async{
     final testFileExists = await PathUtils.correspondingPermanentWidgetTestFile(
            widgetName, filePath)
        .exists();
    filePath = PathUtils.relatetiveLibFilePath(filePath);
      final matchingFunctions = generationsCache.where((element) =>
        element.codeFilePath == filePath &&
        element.widgetName ==widgetName);

    if (matchingFunctions.isEmpty) {
      if (testFileExists) return TestExistStatus.outdatedCodeExists;
      return TestExistStatus.doesNotExist;
    }
    ;
    final exactMatch = matchingFunctions
        .where((element) => element.codeHash == uniqueCodeHex(code));
    if (exactMatch.isEmpty) return TestExistStatus.outdatedCodeExists;
    if (!testFileExists) return TestExistStatus.onlyRequestExists;
    return TestExistStatus.generationExists;
      

  }

  Future<TestExistStatus> testExistStatus(
    String code,
    String filePath,
    String functionName, {
    String? className,
  }) async {
    final testFileExists = await PathUtils.correspondingPermanentTestFile(
            functionName, File(filePath))
        .exists();

    filePath = PathUtils.relatetiveLibFilePath(filePath);
    final matchingFunctions = generationsCache.where((element) =>
        element.codeFilePath == filePath &&
        element.className == className &&
        element.methodName == functionName);

    if (matchingFunctions.isEmpty) {
      if (testFileExists) return TestExistStatus.outdatedCodeExists;
      return TestExistStatus.doesNotExist;
    }
    ;
    final exactMatch = matchingFunctions
        .where((element) => element.codeHash == uniqueCodeHex(code));
    if (exactMatch.isEmpty) return TestExistStatus.outdatedCodeExists;
    if (!testFileExists) return TestExistStatus.onlyRequestExists;
    return TestExistStatus.generationExists;
  }
}

class CodeCache {
  final String codeHash;
  final String codeFilePath;
  final String testHash;
  final String testFilePath;
  final String? className;
  final String? methodName;
  final String? widgetName;
  final bool isSaved;

  CodeCache({
    required this.codeHash,
    required this.codeFilePath,
    required this.testHash,
    required this.testFilePath,
    this.methodName,
    this.widgetName,
    this.isSaved = false,
    this.className,
  });

  factory CodeCache.fromJson(
    String codeHash,
    Map<String, dynamic> json,
  ) =>
      CodeCache(
        codeHash: codeHash,
        codeFilePath: json['codeFilePath'],
        testHash: json['testHash'],
        testFilePath: json['testFilePath'],
        className: json['className'],
        methodName: json['functionName'],
        widgetName: json['widgetName'],
        isSaved: json['saved'],
      );

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['codeFilePath'] = codeFilePath;
    data['testHash'] = testHash;
    data['testFilePath'] = testFilePath;
    data['saved'] = isSaved;
    if (className != null) data['className'] = className;
    if (methodName != null) data['functionName'] = methodName;
    if (widgetName != null) data['widgetName'] = widgetName;

    return data;
  }
}
